MAIN_PAGE_NAME   = "Zelda 2 automap/main.html"

-- ==============================================================================================
-- INTERNAL VARIABLES FOR MISC USE
-- ==============================================================================================

MAP_WIDTH = 64
MAP_HEIGHT = 75

SAVE_FILENAME = "Zelda2Sidebar-visited.dat"

visitedTiles = {}
currentMap = 0

saveCountdown = 0

-- ==============================================================================================
-- ==============================================================================================
-- This runs once before doLogic ever runs
function init()
    resize(335, 430)
	loadHTML(MAIN_PAGE_NAME)
	callJSFunction("clearAllData")
    
    math.randomseed( os.time() )
    currentMap = 0
    for map=0,3,1
    do
        visitedTiles[map] = {}
        for i=0,MAP_WIDTH * MAP_HEIGHT,1
        do
            visitedTiles[map][i] = 15
        end
    end
    
    doLoad()
end


-- ==============================================================================================
-- ==============================================================================================
-- This handles messages sent by the emulator, like if the player presses certain buttons or
-- other events happen within the emulator itself
function getMessage(str)
	if str == "resetgame" then
		init()
	elseif str == "loadstate" then
		loadHTML(MAIN_PAGE_NAME)
		callJSFunction("clearAllData")
	end
end


-- ==============================================================================================
-- ==============================================================================================
-- All main logic goes in here & gets called once per CPU cycle
function doLogic()
	
end

function doLogicFrame()
    if saveCountdown > 0 then
        saveCountdown = saveCountdown - 1
        
        if saveCountdown <= 0 then
            doSave()
        end
    end
end

-- ==================================================================================================
-- MAIN LOGIC (SCRIPT DISPLAYING, ETC.)
-- ==================================================================================================

function getGameState()
    return getMemory(0x736, 1)
end

function sendMap()
    --debug_file = io.open("debug.txt", "wb")
    
    map_string = ""
    address = 0x7c00
    tile_index = 0
    
    while true
    do
        if tile_index > MAP_WIDTH * MAP_HEIGHT or address > 0x7fff then
            break
        end
        
        value = getMemory(address, 1)
        tile = (value & 0xf)
        count = (value & 0xf0) >> 4
        
        if tile == 0 then
            tile = 16
        end
        
        for i=0, count, 1
        do
            map_string = map_string .. string.char(tile)
        end
        
        address = address + 1
        tile_index = tile_index + count
    end
    
    
    --debug_file:write(map_string)
    --debug_file:close()
    
    callJSFunction_Str("updateMapData", map_string)
end

function sendFog()
    --debug_file = io.open("debug.txt", "wb")

    fog_string = ""

    for i=0, MAP_WIDTH * MAP_HEIGHT, 1
    do
        fog_string = fog_string .. string.char(visitedTiles[currentMap][i])
    end
    
    --debug_file:write(fog_string)
    --debug_file:close()
    
    callJSFunction_Str("updateFogData", fog_string)
end

function sendLocation()
    x = getMemory(0x74, 1)
    y = getMemory(0x73, 1) - 30
    
    callJSFunction_Int2("updatePosition", x, y);
    
    if visitedTiles[currentMap][y * MAP_WIDTH + x] ~= 1 then
        visitedTiles[currentMap][y * MAP_WIDTH + x] = 1
        saveCountdown = 30 * 60
    end
end

function doSave()
    out_file = io.open(SAVE_FILENAME, "w+b")
    for map=0, 3, 1
    do
        for i=0, MAP_WIDTH * MAP_HEIGHT, 1
        do
            out_file:write(string.char(visitedTiles[map][i]))
        end
    end
    
    out_file:close()
end

function doLoad()
    in_file = io.open(SAVE_FILENAME, "rb")
    if in_file ~= nil then
        for map=0, 3, 1
        do
            for i = 0, MAP_WIDTH * MAP_HEIGHT - 1, 1
            do
                visitedTiles[map][i] = string.byte(in_file:read(1))
            end
        end
        in_file:close()
    end
end

function PC_CDF5()
    if getBank() == 0x07 and getMemory(0x02, 1) ~= 0xff then
        sendMap()
        
        currentMap = getMemory(0x706, 1)
        sendFog()
    end
end

function PC_DF01()
    if getBank() == 0x07 then
        sendLocation()
    end
end
